/* *****************************************************************************
 * SaFariPark SFP+ editor and support libraries
 * Copyright (C) 2017 National Institute for Subatomic Physics Nikhef
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package nl.nikhef.sfp;

import java.io.IOException;
import java.util.List;

import nl.nikhef.rebus.ftdi.FTDIDevice;
import nl.nikhef.rebus.ftdi.FTDIEnumerator;
import nl.nikhef.rebus.ftdi.FTDIType;

public class MultiSFPProvider extends SFPProviderBase
{
	
	@Override
	public String getName() {
		return "MultiSFP";
	}

	public SFPDevice getDeviceBySerial(String serial) {
		for (SFPDevice dev : getDevices()) {
			if (dev.getSerial().equals(serial)) return dev;
		}
		return null;
	}
	

	public void scanForNewDevices() {
		try {
			List<FTDIDevice> devs;
			synchronized(this)
			{
				
				devs = FTDIEnumerator.getDefault().getDevices();
			}
			
			for (FTDIDevice devRaw : devs)
			{
				
				// System.out.println("Device: " + devRaw);

				// XXX On Linux the MultiSFP A is descriptor is not generated once opened. But B remains. So we sync on that.
				//
				if (devRaw.getType() == FTDIType.FT_2232H && devRaw.getDescription().equals("MultiSFP B")) {
					
					// Only get the secondary 'B' device, we know the first being A
					String serial = devRaw.getSerial();
					
					// chop of the last part from the serial:
					serial = serial.substring(0, serial.length() - 1);
					
					SFPDevice dev = getDeviceBySerial(serial);
					
					
					if (dev == null) 
					{
						dev = new MultiSFPDevice(serial, this);
					}
					updAdd(dev);
				}
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		}
		updProcess();		
	}
	
	

}
